#if TEXTURE_EXT0_ENABLED == 1 || TEXTURE_EXT1_ENABLED == 1 || TEXTURE_EXT2_ENABLED == 1
#extension GL_OES_EGL_image_external : require
#endif

#ifdef _GL_FRAGMENT_PRECISION_HIGH
	precision highp float;
#else
	precision mediump float;
#endif

const int GL_FASTEST = 0x1101;
const int GL_NICEST = 0x1102;

#ifndef TEXTURE0_ENABLED
	#define TEXTURE0_ENABLED -1
#endif

#ifndef TEXTURE1_ENABLED
	#define TEXTURE1_ENABLED -1
#endif

#ifndef TEXTURE2_ENABLED
	#define TEXTURE2_ENABLED -1
#endif

#ifndef FOG_ENABLED
	#define FOG_ENABLED -1
#endif

#ifndef FOG_HINT
	#define FOG_HINT -1
#endif

#ifndef LIGHTING_ENABLED
	#define LIGHTING_ENABLED -1
#endif

#ifndef ALPHA_TEST_ENABLED
	#define ALPHA_TEST_ENABLED -1
#endif

#ifndef CLIP_PLANE0_ENABLED
	#define CLIP_PLANE0_ENABLED -1
#endif

#ifndef CLIP_PLANE1_ENABLED
	#define CLIP_PLANE1_ENABLED -1
#endif

#ifndef CLIP_PLANE2_ENABLED
	#define CLIP_PLANE2_ENABLED -1
#endif

#ifndef CLIP_PLANE3_ENABLED
	#define CLIP_PLANE3_ENABLED -1
#endif

#ifndef CLIP_PLANE4_ENABLED
	#define CLIP_PLANE4_ENABLED -1
#endif

#ifndef CLIP_PLANE5_ENABLED
	#define CLIP_PLANE5_ENABLED -1
#endif

// Uniforms
uniform bool u_fogEnabled;
uniform vec3 u_fogColor;
uniform int u_fogHint;
uniform bool u_alphaTestEnabled;
uniform bool u_lightingEnabled;
uniform vec4 u_globalColor;

// Varyings
varying vec4 v_frontColor;
varying float v_fogFactor;
varying float v_eyeDistance;

// Shader constants
const int c_zero = 0;
const int c_one = 1;
const float c_zerof = 0.0;
const float c_onef = 1.0;

// Funtions
float calcFogFactor(float distanceToEye);
void calcLighting(out vec4 color);
void clipPlanesTest();
void alphaTest(float alpha);
void calcTextureColorContribution(inout vec4 color);


void main() 
{
#if CLIP_PLANE0_ENABLED != 0 || CLIP_PLANE1_ENABLED != 0 || CLIP_PLANE2_ENABLED != 0 || CLIP_PLANE3_ENABLED != 0 || CLIP_PLANE4_ENABLED != 0 || CLIP_PLANE5_ENABLED != 0
	clipPlanesTest();
#endif
	
	vec4 color;
#if LIGHTING_ENABLED == 1
  calcLighting(color);
#else
  color = v_frontColor;
#endif
	
#if TEXTURE0_ENABLED != 0 || TEXTURE1_ENABLED != 0 || TEXTURE2_ENABLED != 0
	calcTextureColorContribution(color);
#endif

#if ALPHA_TEST_ENABLED == 1
  alphaTest(color.a);
#endif
	
#if FOG_ENABLED == 1
  float fogFactor;
  if (u_fogHint == GL_FASTEST) {
		fogFactor = v_fogFactor;
	} else {
		fogFactor = calcFogFactor(v_eyeDistance);
	}
  color.rgb = mix(u_fogColor, color.rgb, fogFactor);
#endif

#ifndef COLOR_ENABLED
  color *= u_globalColor;
#endif
	gl_FragColor = color;
}
