/* pmx/model.vsh */
#ifdef GL_ES
precision highp float;
#endif
#if __VERSION__ < 130
#define in attribute
#define out varying
#endif

uniform mat4 modelViewMatrix;
uniform mat4 modelViewProjectionMatrix;
uniform mat4 lightViewProjectionMatrix;
uniform mat4 lightViewProjectionBiasMatrix;
uniform mat4 normalMatrix;
uniform vec3 lightDirection;

uniform bool hasToonTexture;
uniform bool hasDepthTexture;

in vec3 inPosition;
in vec3 inNormal;
in vec2 inTexCoord;
//in vec4 inUVA1;

out vec2 outMainTexCoord;
out vec4 outShadowTexCoord;
out vec4 outDepth;
//out vec4 outUVA1;
out vec3 outNormal;
out vec3 outEyeNormal;
out vec3 outEyeView;

const float kOne = 1.0;
const float kZero = 0.0;
const vec4 kOne4 = vec4(kOne, kOne, kOne, kOne);
const vec4 kZero4 = vec4(kZero, kZero, kZero, kZero);

void main() {
    vec4 position = vec4(inPosition, kOne);
    
    outNormal = inNormal;
    outEyeNormal = (normalMatrix * vec4(outNormal, kOne)).xyz;
    outEyeView = -(modelViewMatrix * position).xyz;
    
    outMainTexCoord = inTexCoord;

//    outUVA1 = inUVA1;

#ifdef HAS_TOON_TEXTURE
#ifdef HAS_DEPTH_TEXTURE
            outDepth = lightViewProjectionMatrix * position;
            outDepth /= 1000.0;
            outShadowTexCoord = lightViewProjectionBiasMatrix * position;
#endif
#endif

    gl_Position = modelViewProjectionMatrix * position;
}

