/* pmx/edge.vsh */
#ifdef GL_ES
precision highp float;
#endif

#if __VERSION__ < 130
#define in attribute
#define out varying
#endif

invariant gl_Position;

const float kOne  = 1.0;
const int kMaxBones = 100;

uniform mat4 modelViewProjectionMatrix;
uniform vec4 color;
uniform float edgeSize;
uniform vec3 bonePosition[kMaxBones];
uniform vec4 boneRotation[kMaxBones];

in float inType;
in vec3 inPosition;
in vec3 inNormal;
in vec4 inBoneIndices;
in vec4 inBoneWeights;

out vec4 outColor;

struct RETVAL {
   vec3 position;
   vec3 normal;
};

vec3 qtransform(vec4 q, vec3 v)
{
    return v + 2.0 * cross(cross(v, q.xyz) - q.w*v, q.xyz);
}

void performSkinningOne(const float offset, const float weight, const vec3 position, const vec3 normal, inout RETVAL ret)
{
    int boneID = int(offset);
    ret.position += weight * (qtransform(boneRotation[boneID], position) + bonePosition[boneID]);
    ret.normal += weight * qtransform(boneRotation[boneID], normal);
}

void performSkinning(const vec3 position, const vec3 normal, const int type, inout RETVAL ret)
{
    ret.position = vec3(0.0, 0.0, 0.0);
    ret.normal = vec3(0.0, 0.0, 0.0);

    performSkinningOne(inBoneIndices.x, inBoneWeights.x, position, normal, ret);
    performSkinningOne(inBoneIndices.y, inBoneWeights.y, position, normal, ret);
    performSkinningOne(inBoneIndices.z, inBoneWeights.z, position, normal, ret);
    performSkinningOne(inBoneIndices.w, inBoneWeights.w, position, normal, ret);
    
    ret.normal = normalize(ret.normal);
}

void main()
{
    RETVAL ret;
    performSkinning(inPosition, inNormal, int(inType), ret);
    outColor = color;
    gl_Position = modelViewProjectionMatrix * vec4(ret.position + ret.normal * edgeSize, kOne);
}
