/* pmx/model.vsh */
#ifdef GL_ES
precision highp float;
#endif

#if __VERSION__ < 130
#define in attribute
#define out varying
#endif

invariant gl_Position;

const float kOne  = 1.0;
const float kZero = 0.0;
const vec4 kOne4 = vec4(kOne, kOne, kOne, kOne);
const vec4 kZero4 = vec4(kZero, kZero, kZero, kZero);
const int kMaxBones = 100;

uniform mat4 modelViewMatrix;
uniform mat4 modelViewProjectionMatrix;
uniform mat4 lightViewProjectionMatrix;
uniform mat4 lightViewProjectionBiasMatrix;
uniform mat4 normalMatrix;
uniform vec3 lightDirection;
uniform bool hasDepthTexture;
uniform vec3 bonePosition[kMaxBones];
uniform vec4 boneRotation[kMaxBones];

in float inType;
in vec3 inPosition;
in vec3 inNormal;
in vec2 inTexCoord;
//in vec4 inUVA1;
in vec4 inBoneIndices;
in vec4 inBoneWeights;

out vec2 outMainTexCoord;
out vec4 outShadowTexCoord;
out vec4 outDepth;
//out vec4 outUVA1;
out vec3 outNormal;
out vec3 outEyeNormal;
out vec3 outEyeView;

struct RETVAL {
   vec3 position;
   vec3 normal;
};

vec3 qtransform(vec4 q, vec3 v)
{
    return v + 2.0 * cross(cross(v, q.xyz) - q.w*v, q.xyz);
}

void performSkinningOne(const float offset, const float weight, const vec3 position, const vec3 normal, inout RETVAL ret)
{
    int boneID = int(offset);
    ret.position += weight * (qtransform(boneRotation[boneID], position) + bonePosition[boneID]);
    ret.normal += weight * qtransform(boneRotation[boneID], normal);
}

void performSkinning(const vec3 position, const vec3 normal, const int type, inout RETVAL ret)
{
    ret.position = vec3(0.0, 0.0, 0.0);
    ret.normal = vec3(0.0, 0.0, 0.0);

    performSkinningOne(inBoneIndices.x, inBoneWeights.x, position, normal, ret);
    performSkinningOne(inBoneIndices.y, inBoneWeights.y, position, normal, ret);
    performSkinningOne(inBoneIndices.z, inBoneWeights.z, position, normal, ret);
    performSkinningOne(inBoneIndices.w, inBoneWeights.w, position, normal, ret);
}

void main() {
    RETVAL ret;
    
    performSkinning(inPosition, inNormal, int(inType), ret);
    vec4 position = vec4(ret.position, kOne);
    
    outNormal = ret.normal;
    outEyeNormal = (normalMatrix * vec4(outNormal, kOne)).xyz;
    outEyeView = -(modelViewMatrix * position).xyz;
    
    outMainTexCoord = inTexCoord;
    
//    outUVA1 = inUVA1;

#ifdef HAS_TOON_TEXTURE
#ifdef HAS_DEPTH_TEXTURE
            outDepth = lightViewProjectionMatrix * position;
            outDepth /= 1000.0;
            outShadowTexCoord = lightViewProjectionBiasMatrix * position;
#endif
#endif
    
    gl_Position = modelViewProjectionMatrix * position;
}

