/* pmx/zplot.vsh */
#ifdef GL_ES
precision highp float;
#endif

#if __VERSION__ < 130
#define in attribute
#define out varying
#endif

invariant gl_Position;

in float inType;
in vec3 inPosition;
in vec4 inBoneIndices;
in vec4 inBoneWeights;

out vec4 outPosition;

const int kMaxBones = 100;

uniform mat4 modelViewProjectionMatrix;
uniform vec3 bonePosition[kMaxBones];
uniform vec4 boneRotation[kMaxBones];


vec3 qtransform(vec4 q, vec3 v)
{
    return v + 2.0 * cross(cross(v, q.xyz) - q.w*v, q.xyz);
}

void performSkinningOne(const float offset, const float weight, const vec3 position, inout vec3 outPosition)
{
    int boneID = int(offset);
    outPosition += weight * (qtransform(boneRotation[boneID], position) + bonePosition[boneID]);
}

vec3 performSkinning(const vec3 position, const int type)
{
    vec3 outPosition = vec3(0.0, 0.0, 0.0);
    
    performSkinningOne(inBoneIndices.x, inBoneWeights.x, position, outPosition);
    performSkinningOne(inBoneIndices.y, inBoneWeights.y, position, outPosition);
    performSkinningOne(inBoneIndices.z, inBoneWeights.z, position, outPosition);
    performSkinningOne(inBoneIndices.w, inBoneWeights.w, position, outPosition);
    
    return outPosition;
}

void main() {
    vec4 position = modelViewProjectionMatrix * vec4(performSkinning(inPosition, int(inType)), 1.0);
    outPosition = position / 1000.0 + 0.00005;
    gl_Position = position;
}

