#ifdef GL_ES
precision highp float;
#endif

uniform sampler2D blurTextureMin;
uniform sampler2D blurTextureMax;
uniform sampler2D mainTexture;
uniform sampler2D depthTexture;
uniform float     depthTarget;
uniform float     depthRange;

varying vec2      outTexCoord;

const vec4 unpackMask = vec4(1.0, 1.0 / 255.0, 1.0 / (255.0 * 255.0), 1.0 / (255.0 * 255.0 * 255.0));

float unpack_depth(const vec4 RGBA)
{
  return dot(RGBA, unpackMask);
}

float convCoord(float depth){
	float d = min(depth, 2.0);
	
	d = 1.0 - abs(depth - depthTarget) / depthRange;
	d = clamp(d, 0.0, 1.0);
	
	return d;
}

void main(void)
{
  float d = convCoord(unpack_depth(texture2D(depthTexture, outTexCoord)));
  float coef = 1.0 - d;
  float blur1Coef = coef * d;
  float blur2Coef = coef * coef;
  
  vec4 sceneColor = texture2D(mainTexture, outTexCoord);
  vec4 blur1Color = texture2D(blurTextureMin, outTexCoord);
  vec4 blur2Color = texture2D(blurTextureMax, outTexCoord);
  
  gl_FragColor = sceneColor * d + blur1Color * blur1Coef + blur2Color * blur2Coef;
}