local instance = {}

local mainTexture = MMDAgent.getTexture2D("MAIN")
local depthTexture = MMDAgent.getTexture2D("DEPTH")

local fbo = MMDAgent.createFBO()
fbo:attachTexture(0, mainTexture)

-- ガウシアンMin
local blurTextureMin = MMDAgent.createScreenTexture()
local blurFBOMin = MMDAgent.createFBO()
blurFBOMin:attachTexture(0, blurTextureMin)
local blurMin = dofile(MMDAgent.upDir(LOCAL_PATH).."/Gaussian/gaussian.lua")
blurMin.setTexture(mainTexture)
-- ガウシアンMax
local blurTextureMax = MMDAgent.createScreenTexture()
local blurFBOMax = MMDAgent.createFBO()
blurFBOMax:attachTexture(0, blurTextureMax)
local blurMax = dofile(MMDAgent.upDir(LOCAL_PATH).."/Gaussian/gaussian.lua")
blurMax.setTexture(mainTexture)
-- 合成
local targetDepth = 0
local program = MMDAgent.createShaderProgram()
program.attributes = {inPosition = 0, inTexCoord = 1}
program.uniforms = {mainTexture = -1, depthTexture = -1, blurTextureMin = -1, blurTextureMax = -1, depthTarget = -1, depthRange = -1}
program:build(readfile(LOCAL_PATH.."/dof.vsh"), readfile(LOCAL_PATH.."/dof.fsh"))


-- 設定変更
local rangeMaxTemp = 0
local targetDepthMin = 0
local targetDepthMax = 0
local targetDepthMaxTemp = 0

instance.setBlurRangeMin = function(value)
  blurMin.setRange(value)
  blurMax.setRange(value + rangeMaxTemp)
end

instance.setBlurRangeMax = function(value)
  rangeMaxTemp = value
  blurMax.setRange(blurMin.getRange() + rangeMaxTemp)
end

instance.setTargetDepth = function(value)
  targetDepthMin = value
  targetDepthMax = targetDepthMin + targetDepthMaxTemp
end

instance.setTargetDepthRange = function(value)
  targetDepthMaxTemp = value
  targetDepthMax = targetDepthMin + targetDepthMaxTemp
end

instance.render = function()
  -- ガウシアンMin
  blurFBOMin:bind()
  gl.ClearColor()
  blurMin.render()
  blurFBOMin:unbind()
  -- ガウシアンMax
  blurFBOMax:bind()
  gl.ClearColor()
  blurMax.render()
  blurFBOMax:unbind()
  
  -- 以下合成
  
  gl.Disable(gl.BLEND)
  
  -- bind
  fbo:bind()
  program:bind()
  local vao = MMDAgent.getScreenVAO()
  vao:bind()
  
  -- uniform
  MMDAgent.setUniformTexture(program.uniforms.mainTexture, 0, mainTexture)
  MMDAgent.setUniformTexture(program.uniforms.depthTexture, 1, depthTexture)
  MMDAgent.setUniformTexture(program.uniforms.blurTextureMin, 2, blurTextureMin)
  MMDAgent.setUniformTexture(program.uniforms.blurTextureMax, 3, blurTextureMax)
  gl.Uniform1(program.uniforms.depthTarget, targetDepthMin)
  gl.Uniform1(program.uniforms.depthRange, targetDepthMax)
  
  -- draw
  vao.drawScreenElements()
  
  -- unbind
  vao:unbind()
  program:unbind()
  fbo:unbind()
  
  gl.Enable(gl.BLEND)
end

return instance