-- 変数
curPoseFile = {}
local selectModel
local selectBone
local selectModelRender

-- ステージファイルが選択された
local function stageFileSelected(alias, file)
  save["stage"] = file
  MMDAgent.stage(file)
end

-- モデルファイルが選択された
local function modelFileSelected(alias, file)
  ProgressDialog.start("now loading a model...")
  save["alias_counter"] = save["alias_counter"] or 0
  MMDAgent.addModel("actor"..save["alias_counter"], file, Vector3.New(0, 0, 0))
  save["alias_counter"] = save["alias_counter"] + 1
  ProgressDialog.delete()
end

-- スクリプトファイルが選択された
local function scriptFileSelected(alias, file)
  local script = dofile(file)
  if (save['select_model'] and script.setModel) then
    script.setModel(save['select_model'])
  end
end

-- ARカメラ
function changeARCamera(use)
  save["arcamera"]["use"] = use
  if (use) then
    ARCamera.open()
    ARCamera.start()
  else
    ARCamera.close()
  end
end

-- セルフシャドウ
function changeSelfShadow(use)
  ProgressDialog.start("now applying changes...")
  
  MMDAgent.selfShadow.use(use)
  save['selfshadow']['use'] = use
  
  ProgressDialog.delete()
end

-- 終了時にセーブする
function POSE_SAVE_FILE(alias)
  return "Pose/save_"..alias..".vpd"
end

local function windowClose()
  -- カメラ状態
  save["camera"] = MMDAgent.getCameraInfo()
  -- ポーズ状態
  for alias, model in pairs(save['models']) do
    MMDAgent.savePose(alias, POSE_SAVE_FILE(alias))
  end
  
  TableLoader.store("save.dat", save)
end

-- 着地
function landing()
  local bone = MMDAgent.getBoneInfo(save['select_model'], '全ての親', false)
  local xyz = Vector3.New(bone['x'], 0, bone['z'])
  MMDAgent.moveBone(save['select_model'], '全ての親', xyz, true)
  MMDAgent.sendMessage("BONE_EVENT_UPDATE", save['select_model']..'|全ての親')
end

-- ポーズクリア
function clearPose()
  MMDAgent.deleteMotion(save['select_model'])
end

-- ポーズセーブ
function saveAsPose()
  Filer.save('SAVE_POSE', 'Pose', '', '.vpd')
end

function savePose(file)
  if (file) then
    MMDAgent.savePose(save['select_model'], file)
  else
    saveAsPose()
  end
end

function savePoseFromFiler(alias, file)
  curPoseFile[save['select_model']] = file
  savePose(file)
end

-- ポーズ追加
local function poseFileSelected(alias, file)
  MMDAgent.deleteMotion(save['select_model'], 'pose')
  MMDAgent.addMotion(save['select_model'], file, file, MMDAgent.FULL, MMDAgent.LOOP, MMDAgent.ON, MMDAgent.OFF, 0, 2)
end

-- 変更するモデルファイルを選択した
local function changeModelFileSelected(alias, file)
  ProgressDialog.start("now loading a model...")
  MMDAgent.assignModel("HANDLE")
  MMDAgent.changeModel(save['select_model'], file)
  ProgressDialog.delete()
end

-- モデルが追加された
local function modelAdded(alias, file)
  if (save["models"][alias]) then
    MMDAgent.addMotion(alias, POSE_SAVE_FILE(alias), POSE_SAVE_FILE(alias), MMDAgent.FULL, MMDAgent.LOOP, MMDAgent.ON, MMDAgent.OFF, 0)
  else
    save["models"][alias] = {}
  end
  save["models"][alias]["file"] = file
  
  ShowDocument('System/UI/touch-menu-control.rml')
end

-- モデルが変更された
local function modelChanged(alias, file)
  save['models'][alias]['file'] = file
  MMDAgent.sendMessage("FOCUS_EVENT_CHANGE", alias)
end

-- モデルが削除された
local function modelDeleted(alias)
  -- 要素削除
  save['models'][alias] = nil 
  selectModel = nil
  selectBone = nil
  selectModelRender = nil
  
  -- 一時ポーズファイル削除
  os.remove(POSE_SAVE_FILE(alias))
  
  if (0 == table.elemn(save['models'])) then
    ShowDocument('System/UI/touch-menu-control.rml'):Hide()
  end
end

-- モデル削除
function modelDelete()
  MMDAgent.assignModel("HANDLE")
  MMDAgent.deleteModel(save['select_model'])
end

-- ハンドル操作
local BOXHALFSIZE = MMDAgent.getPixelsPerInch() * 0.05

local HANDLE_EMPTY  = 0
local HANDLE_X_ROT = 1
local HANDLE_Y_ROT = 2
local HANDLE_Z_ROT = 3
local HANDLE_X_POS = 4
local HANDLE_Y_POS = 5
local HANDLE_Z_POS = 6

local handleState = HANDLE_EMPTY
local oldMouseWorldPos
local hadleRigidBodys = {}
local handleModel
local handleRender

local function handleMouseLeftDown()
  if (false == MMDAgent.getControl()) then
    return
  end
  
  local x, y = MMDAgent.getMousePosition()
  local startPos = MMDAgent.getScreen2WorldPosition(x, y, 0)
  local endPos = MMDAgent.getScreen2WorldPosition(x, y, 1)
  
  local hitBody = MMDAgent.rayTestByTable(startPos, endPos, hadleRigidBodys)
  if (hitBody == nil) then
    return
  end
  
  local name = hitBody:getName()
  if (name == "X回転") then
    handleState = HANDLE_X_ROT
  elseif (name == "Y回転") then
    handleState = HANDLE_Y_ROT
  elseif (name == "Z回転") then
    handleState = HANDLE_Z_ROT
  elseif (name == "X移動") then
    handleState = HANDLE_X_POS
  elseif (name == "Y移動") then
    handleState = HANDLE_Y_POS
  elseif (name == "Z移動") then
    handleState = HANDLE_Z_POS
  end
  
  if (handleState ~= HANDLE_EMPTY) then
    MMDAgent.control(false)
    oldMouseWorldPos = MMDAgent.getScreen2WorldPosition(x, y, 0.5)
  end
end

local function handleMouseLeftUp()
  if (handleState == HANDLE_EMPTY) then
    return
  end
  
  MMDAgent.control(true)
  handleState = HANDLE_EMPTY
end

local function handleUpdate()
  if (handleState == HANDLE_EMPTY) then
    return
  end
  
  local x, y = MMDAgent.getMousePosition()
--[[
  local bonePos = handleModel:getBone("全ての親"):getWorldPosition()
  local camera = MMDAgent.getCamera()
  local cameraPos = camera:getPosition():rotate(camera::getAngle())
--]]
  local pos = MMDAgent.getScreen2WorldPosition(x, y, 0.5)
  
  if (handleState == HANDLE_X_POS) then
    MMDAgent.moveBone(save['select_model'], save['select_bone'], Vector3.New(pos.x- oldMouseWorldPos.x, 0, 0), false)
  elseif (handleState == HANDLE_Y_POS) then
    MMDAgent.moveBone(save['select_model'], save['select_bone'], Vector3.New(0, pos.y - oldMouseWorldPos.y, 0), false)
  elseif (handleState == HANDLE_Z_POS) then
    MMDAgent.moveBone(save['select_model'], save['select_bone'], Vector3.New(0, 0, pos.z - oldMouseWorldPos.z), false)
  elseif (handleState == HANDLE_X_ROT) then
    MMDAgent.rotateBone(save['select_model'], save['select_bone'], Vector3.New(oldMouseWorldPos.y - pos.y, 0, 0) * 180, false)
  elseif (handleState == HANDLE_Y_ROT) then
    MMDAgent.rotateBone(save['select_model'], save['select_bone'], Vector3.New(0, pos.x - oldMouseWorldPos.x, 0) * 180, false)
  elseif (handleState == HANDLE_Z_ROT) then
    MMDAgent.rotateBone(save['select_model'], save['select_bone'], Vector3.New(0, 0, oldMouseWorldPos.x - pos.x) * 180, false)
  end
  
  oldMouseWorldPos = pos
end

-- ボーン選択
local function boneMouseLeftDown()
  if (false == MMDAgent.getControl()) then
    return
  end
  if (handleState ~= HANDLE_EMPTY) then
    return
  end
  
  local x, y = MMDAgent.getMousePosition()
  local model = MMDAgent.getModel(save['select_model'])
  local bones = model:getBones()
  for i, bone in pairs(bones) do
    local boneX, boneY = MMDAgent.getWorld2ScreenPosition(bone:getWorldPosition())
    
    if (boneX - BOXHALFSIZE <= x and boneX + BOXHALFSIZE >= x) then
      if (boneY - BOXHALFSIZE <= y and boneY + BOXHALFSIZE >= y) then
        MMDAgent.sendMessage("BONE_EVENT_SELECT", bone:getName())
        break
      end
    end
  end
end

local function changeFocus(alias)
  selectModel = MMDAgent.getModel(alias)
  selectModelRender = selectModel:getRenderEngine()
end
local function changeFocusBone(name)
  selectBone = selectModel:getBone(name)
end

local function renderBone()
  if (nil == selectBone) then
    return
  end
  
  selectModelRender:renderBone()
end

local pointProgram = MMDAgent.createShaderProgram()
pointProgram.attributes = {inPosition = 0}
pointProgram.uniforms = {modelViewProjectionMatrix = -1, pointSize = -1}
pointProgram:build(readfile("System/Shader/bone.vsh"), readfile("System/Shader/bone.fsh"))

local function renderHandle()
  gl.Disable(gl.DEPTH_TEST)
  
  -- ボーンの色を変える
  local pos = selectBone:getWorldPosition()
  
  pointProgram:bind()
  gl.EnableVertexAttribArray(pointProgram.attributes.inPosition)
  gl.VertexAttribPointer(pointProgram.attributes.inPosition, {pos.x, pos.y, pos.z}, 3)
  MMDAgent.setUniformMatrix(pointProgram.uniforms.modelViewProjectionMatrix, MMDAgent.WORLD_MATRIX | MMDAgent.VIEW_MATRIX | MMDAgent.PROJECTION_MATRIX | MMDAgent.CAMERA_MATRIX)
  gl.Uniform1(pointProgram.uniforms.pointSize, BOXHALFSIZE)
  gl.Enable(gl.VERTEX_PROGRAM_POINT_SIZE)
  gl.DrawArrays(gl.POINTS, 0, 1)
  gl.Disable(gl.VERTEX_PROGRAM_POINT_SIZE)
  gl.DisableVertexAttribArray(pointProgram.attributes.inPosition)
  pointProgram:unbind()
  
  -- ハンドル
  handleModel:setVisible(true)
  handleRender:update()
  handleRender:renderModel()
  handleModel:setVisible(false)
  
  gl.Enable(gl.DEPTH_TEST)
end

function checkShowBones(enable)
  if (tonumber(enable)) then
    MMDAgent.addRenderListener("SHOW_BONES", renderBone, 2000)
  else
    MMDAgent.deleteRenderListeners("SHOW_BONES")
  end
end

function checkShowHandle(enable)
  if (tonumber(enable)) then
    MMDAgent.addMessageListener("SHOW_HANDLE", "MOUSE_EVENT_LEFTDOWN", "", handleMouseLeftDown, 0)
    MMDAgent.addMessageListener("SHOW_HANDLE", "MOUSE_EVENT_LEFTUP", "", handleMouseLeftUp, 0)
    MMDAgent.addMessageListener("SHOW_HANDLE", "MOUSE_EVENT_LEFTDOWN", "", boneMouseLeftDown, 10)
    MMDAgent.addRenderListener("SHOW_HANDLE", handleUpdate, 2000 + 10)
    MMDAgent.addRenderListener("SHOW_HANDLE", renderHandle, 2000 + 10)
  else
    MMDAgent.deleteMessageListeners("SHOW_HANDLE")
    MMDAgent.deleteRenderListeners("SHOW_HANDLE")
  end
end

-- 各種ロード
local function systemLoad()
  -- メッセージリスナーの登録
  MMDAgent.addMessageListener("", "FILE_EVENT_SELECT", "STAGE", stageFileSelected)
  MMDAgent.addMessageListener("", "FILE_EVENT_SELECT", "MODEL", modelFileSelected)
  MMDAgent.addMessageListener("", "FILE_EVENT_SELECT", "SCRIPT", scriptFileSelected)
  MMDAgent.addMessageListener("", "FILE_EVENT_SELECT", "POSE", poseFileSelected)
  MMDAgent.addMessageListener("", "FILE_EVENT_SELECT", "MODEL_CHANGE", changeModelFileSelected)
  MMDAgent.addMessageListener("", "FILE_EVENT_SELECT", "SAVE_POSE", savePoseFromFiler)
  MMDAgent.addMessageListener("", "MODEL_EVENT_ADD", "", modelAdded)
  MMDAgent.addMessageListener("", "MODEL_EVENT_CHANGE", "", modelChanged)
  MMDAgent.addMessageListener("", "MODEL_EVENT_DELETE", "", modelDeleted)
  MMDAgent.addMessageListener("", "FOCUS_EVENT_CHANGE", "", changeFocus)
  MMDAgent.addMessageListener("", "BONE_EVENT_SELECT", "", changeFocusBone)
  MMDAgent.addMessageListener("", "PLUGIN_DISABLE", "Lua", windowClose)
  
  -- セーブデータ
  save = TableLoader.load("save.dat")
  save = save or {}
  save["models"] = save["models"] or {}
  
  -- UI
  ShowDocument('System/UI/touch-menu.rml')
  ShowDocument('System/UI/touch-capture.rml')
  ShowDocument('System/UI/touch-other.rml'):Hide()
  
  ShowDocument('System/UI/wholeSettingsWindow.rml'):Hide()
  ShowDocument('System/UI/lightWindow.rml'):Hide()
  ShowDocument('System/UI/cameraWindow.rml'):Hide()
  ShowDocument('System/UI/effect.rml'):Hide()
  
  -- ステージの復元
  if (save['stage']) then
    MMDAgent.stage(save['stage'])
  end
  
  -- モデルの復元
  for alias,model in pairs(save["models"]) do 
    MMDAgent.addModel(alias, model["file"], Vector3.Zero())
  end
  
  -- ウィンドウ
  ShowDocument('System/UI/targetModelWindow.rml'):Hide()
  ShowDocument('System/UI/targetBoneWindow.rml'):Hide()
  
  ProgressDialog.delete()
end

local function resourceLoaded()
  MMDAgent.addMessageListener("", "SYSTEM_EVENT_LOAD", "", systemLoad)
  MMDAgent.sendMessage("SYSTEM_EVENT_LOAD", "")
  
  -- ハンドルの剛体を作成
  handleModel = MMDAgent.getModel("HANDLE")
  handleModel:setVisible(false)
  handleRender = handleModel:getRenderEngine()
  local boneList = {"X回転", "Y回転", "Z回転", "X移動", "Y移動", "Z移動"}
  for i, name in pairs(boneList) do
    local bone = handleModel:getBone(name)
    local body = bone:createConvexHullRigidBody(name, 0, 0)
    table.insert(hadleRigidBodys, body)
  end
end

function init(doc)
  MMDAgent.addMessageListener("", "MODEL_EVENT_ADD", "HANDLE", resourceLoaded)
  
  MMDAgent.addModel("HANDLE", "System/Model/Handle.pmx")
end