Vector3 = dofile('System/lua/Vector3.lua')
Quaternion = dofile('System/lua/Quaternion.lua')

MMDAgent = MMDAgent or {}

-- テーブルに実際に存在する要素数(=num of elements)
-- 「#」や「table.getn」「table.maxn」ではテーブルの要素数は調べられないので必要となる。
function table.elemn(tbl)
    local n = 0
    for _ in pairs (tbl) do
        n = n + 1
    end
    return n
end

function toboolean(X)
	return not not X
end

MMDAgent.MODEL_FORMAT = [[.x;.lwo;.dxf;.3ds;.pmd;.pmx]]

local function vector32String(v)
  return v.x..","..v.y..","..v.z
end

-- 定数
MMDAgent.FULL = "FULL"
MMDAgent.PART = "PART"
MMDAgent.ONCE = "ONCE"
MMDAgent.LOOP = "LOOP"
MMDAgent.ON = "ON"
MMDAgent.OFF = "OFF"

-- ログ表示
MMDAgent.log = {}
function MMDAgent.log.use(flag)
  if toboolean(flag) then
    MMDAgent.sendMessage("KEY", "D")
  else
    MMDAgent.sendMessage("KEY", "D")
  end
end

-- 軸表示
MMDAgent.axis = {}
function MMDAgent.axis.use(flag)
  MMDAgent.sendMessage("AXIS", flag)
end

-- コントロールの有効化/無効化
function MMDAgent.control(flag)
  if toboolean(flag) then
    MMDAgent.sendMessage("CONTROLL_ENABLE", "")
  else
    MMDAgent.sendMessage("CONTROLL_DISABLE", "")
  end
end

-- MMDAgentの再開
function MMDAgent.resume()
  MMDAgent.sendMessage("MMDAGENT_RESUME", "")
end

-- MMDAgentの停止
function MMDAgent.stop()
  MMDAgent.sendMessage("MMDAGENT_STOP", "")
end

-- プラグインの有効化/無効化
MMDAgent.plugin = {}
function MMDAgent.plugin.use(name, flag)
  if tonumber(flag) then
    MMDAgent.sendMessage("PLUGIN_ENABLE", name)
  else
    MMDAgent.sendMessage("PLUGIN_DISABLE", name)
  end
end

-- ドロップシャドウ
MMDAgent.dropShadow = {}
function MMDAgent.dropShadow.use(use)
  MMDAgent.sendMessage("DROPSHADOW", use)
end

-- セルフシャドウ
MMDAgent.selfShadow = {}
function MMDAgent.selfShadow.use(use)
  MMDAgent.sendMessage("SELFSHADOW", tostring(use))
end

-- セルフシャドウのテクスチャーサイズを設定
function MMDAgent.selfShadow.textureSize(value)
  MMDAgent.sendMessage("SELFSHADOW_TEXTURESIZE", value)
end

-- テクスチャの最大サイズを設定
function MMDAgent.textureMaxSize(value)
  MMDAgent.sendMessage("TEXTURE_MAXSIZE", value)
end

-- 物理演算のフレームスキップ設定
MMDAgent.bullet = {}
function MMDAgent.bullet.frameskip(value)
  value = value or '0'
  MMDAgent.sendMessage("BULLET_FRAMESKIP", value)
end

-- 重力設定
function MMDAgent.bullet.gravity(xyz)
  MMDAgent.sendMessage("GRAVITY", vector32String(xyz * -9.8))
end

-- 物理演算FPSの設定
function MMDAgent.bullet.fps(value)
  MMDAgent.sendMessage("BULLET_FPS", value)
end

-- 背景カラー
function MMDAgent.campusColor(rgb)
  MMDAgent.sendMessage("CAMPUS_COLOR", vector32String(rgb))
end

-- ライトカラー
function MMDAgent.lightColor(rgb)
  MMDAgent.sendMessage("LIGHT_COLOR", vector32String(rgb))
end

-- ライトの方向
function MMDAgent.lightDirection(xyz)
  MMDAgent.sendMessage("LIGHT_DIRECTION", vector32String(xyz))
end

-- ライトの強さ
function MMDAgent.lightIntensity(value)
  MMDAgent.sendMessage("LIGHT_INTENSITY", value)
end

-- エッジの太さ
function MMDAgent.edgeSize(value)
  MMDAgent.sendMessage("EDGE_SIZE", value)
end

-- カメラ制御
function MMDAgent.camera(xyz, rxyz, distance, fovy, time)
  time = time or 0
  MMDAgent.sendMessage("CAMERA", vector32String(xyz).."|"..vector32String(rxyz).."|"..distance.."|"..fovy.."|"..time)
end

-- カメラ制御（for VMDファイル）
function MMDAgent.cameraFromMotion(file)
  MMDAgent.sendMessage("CAMERA", file)
end

-- カメラ一時停止
function MMDAgent.pauseCamera()
  MMDAgent.sendMessage("CAMERA_PAUSE")
end

-- カメラ再開
function MMDAgent.resumeCamera()
  MMDAgent.sendMessage("CAMERA_RESUME")
end

-- カメラシーク
function MMDAgent.seekCamera(time)
  MMDAgent.sendMessage("CAMERA_SEEK", tostring(time))
end

-- カメラ停止
function MMDAgent.stopCamera()
  MMDAgent.sendMessage("CAMERA_STOP")
end

-- ステージ
function MMDAgent.stage(file)
  MMDAgent.sendMessage("STAGE", file)
end

-- モデル追加
function MMDAgent.addModel(alias, file, xyz, rxyz)
  xyz = xyz or Vector3.Zero()
  rxyz = rxyz or Vector3.Zero()
  MMDAgent.sendMessage("MODEL_ADD", alias.."|"..file.."|"..vector32String(xyz).."|"..vector32String(rxyz))
end

-- モデル変更
function MMDAgent.changeModel(alias, file)
  MMDAgent.sendMessage("MODEL_CHANGE", alias.."|"..file)
end

-- モデル削除
function MMDAgent.deleteModel(alias)
  MMDAgent.sendMessage("MODEL_DELETE", alias)
end

-- モデル追従
function MMDAgent.assignModel(alias, parent, bone)
  if (parent == nil) then
    MMDAgent.sendMessage("MODEL_ASSIGN", alias)
  else
    MMDAgent.sendMessage("MODEL_ASSIGN", alias.."|"..parent.."|"..bone)
  end
end

-- ボーン移動
function MMDAgent.moveBone(alias, name, xyz, absolute)
  MMDAgent.sendMessage("BONE_MOVE", alias.."|"..name.."|"..vector32String(xyz).."|"..tostring(absolute))
end

-- ボーン回転
function MMDAgent.rotateBone(alias, name, xyz, absolute)
  MMDAgent.sendMessage("BONE_ROTATE", alias.."|"..name.."|"..vector32String(xyz).."|"..tostring(absolute))
end

-- モーフ
function MMDAgent.morph(alias, name, weight, time)
  time = time or 0
  MMDAgent.sendMessage("MORPH", alias.."|"..name.."|"..weight.."|"..time)
end

-- モーション追加
function MMDAgent.addMotion(model, alias, file, frame, loop, smooth, relocation, priority, time)
  frame = frame or MMDAgent.FULL
  loop = loop or MMDAgent.ONCE
  smooth = smooth or MMDAgent.ON
  relocation = relocation or MMDAgent.OFF
  priority = priority or 0
  time = time or 0
  MMDAgent.sendMessage("MOTION_ADD", model.."|"..alias.."|"..file.."|"..frame.."|"..loop.."|"..smooth.."|"..relocation.."|"..priority.."|"..time)
end

-- モーション変更
function MMDAgent.changeMotion(model, alias, file, time)
  time = time or 0
  MMDAgent.sendMessage("MOTION_CHANGE", model.."|"..alias.."|"..file.."|"..time)
end

-- モーション削除
function MMDAgent.deleteMotion(model, alias)
  if (alias) then
    MMDAgent.sendMessage("MOTION_DELETE", model.."|"..alias)
  else
    MMDAgent.sendMessage("MOTION_DELETE", model)
  end
end

-- モーション内ボーン削除
function MMDAgent.deleteBoneFromMotion(model, alias, bone)
  MMDAgent.sendMessage("MOTION_DELETEBONE", model.."|"..alias.."|"..bone)
end

-- モーション内モーフ削除
function MMDAgent.deleteMorphFromMotion(model, alias, morph)
  MMDAgent.sendMessage("MOTION_DELETEMORPH", model.."|"..alias.."|"..morph)
end

-- モーション加速
function MMDAgent.accelerateMotion(model, alias, speed, duration, start)
  MMDAgent.sendMessage("MOTION_ACCELERATE", model.."|"..alias.."|"..speed.."|"..duration.."|"..start)
end

-- モーション一時停止
function MMDAgent.pauseMotion(model, alias)
  if (alias) then
    MMDAgent.sendMessage("MOTION_PAUSE", model.."|"..alias)
  else
    MMDAgent.sendMessage("MOTION_PAUSE", model)
  end
end

-- モーション再開
function MMDAgent.resumeMotion(model, alias)
  if (alias) then
    MMDAgent.sendMessage("MOTION_RESUME", model.."|"..alias)
  else
    MMDAgent.sendMessage("MOTION_RESUME", model)
  end
end

-- モーション再生位置
function MMDAgent.seekMotion(model, alias, to)
  MMDAgent.sendMessage("MOTION_SEEK", model.."|"..alias.."|"..to)
end

-- ポーズの読み込み
function MMDAgent.loadPose(model, file)
  MMDAgent.sendMessage("POSE_LOAD", model.."|"..file)
end

-- ポーズの保存
function MMDAgent.savePose(model, file)
  MMDAgent.sendMessage("POSE_SAVE", model.."|"..file)
end

-- ファイル選択
Filer = Filer or {}
function Filer.select(alias, dir, ext)
  local document = ShowDocument('System/UI/filer-select.rml')
  local filer = filer_instance(document)
  filer:update(alias, dir, ext)
end

-- ファイル保存選択
function Filer.save(alias, dir, default, ext)
  local document = ShowDocument('System/UI/filer-save.rml')
  local filer = filer_instance(document)
  filer:update(alias, dir, default, ext)
end

-- ARカメラの準備
ARCamera = {}
function ARCamera.open(cameraid)
  MMDAgent.sendMessage("ARCAMERA_OPEN", tostring(cameraid))
end

-- ARカメラの開始
function ARCamera.start()
  MMDAgent.sendMessage("ARCAMERA_START", "")
end

-- ARカメラの停止
function ARCamera.stop()
  MMDAgent.sendMessage("ARCAMERA_STOP", "")
end

-- ARカメラのクローズ
function ARCamera.close()
  MMDAgent.sendMessage("ARCAMERA_CLOSE", "")
end

-- サウンド再生
Sound = {}
function Sound.start(alias, file)
  MMDAgent.sendMessage("SOUND_START", alias.."|"..file)
end

-- サウンド停止
function Sound.stop(alias)
  MMDAgent.sendMessage("SOUND_STOP", alias)
end

-- サウンド一時停止
function Sound.pause(alias)
  MMDAgent.sendMessage("SOUND_PAUSE", alias)
end

-- サウンド再開
function Sound.resume(alias)
  MMDAgent.sendMessage("SOUND_RESUME", alias)
end

-- サウンド再生位置
function Sound.seek(alias, time)
  MMDAgent.sendMessage("SOUND_SEEK", alias.."|"..time)
end

-- Twitterへ投稿
Twitter = {}
function Twitter.send(status, file)
  MMDAgent.sendMessage("TWITTER_SEND", status.."|"..file)
end

