local inifile = dofile("System/lua/LIP.lua")

local parser = {}

local function make_param_rml(funcName, key, param)
  local rml = ''
  
  local name = param.name or ""
  
  if ("range" == param.type) then
    local code = funcName..[[(']]..key..[[', ]]..(param.max - param.min)..[[ * tonumber(event.parameters['value'] - ]]..param.min..[[))]]
    rml = [[<p>]]..name..[[<input type="range" onchange="]]..code..[[" id="]]..key..[[" value="]]..param.value..[[" min="]]..param.min..[[" max="]]..param.max..[[" step="]]..(param.step or 1)..[["/></p>]]
  elseif("radio" == param.type) then
    local code = funcName..[[(']]..key..[[', tonumber(event.parameters['value']))]]
    rml = [[<input type="radio" onchange="]]..code..[[" id="]]..key..[[" name="]]..key..[[" value="]]..param.value..[[">]]..name..[[</input>]]
  end
  
  return rml
end

parser.load = function(funcName, file, saveData)
  local out = {info = {}, defaults = {}}
  local ini = {}
  
  if (file_exists(file)) then
    ini = inifile.load(file)
  end
  
  if (table.elemn(ini) == 0) then
    return out
  end
  
  -- parse 'info'
  out.info = ini.info
  ini.info = nil
  
  -- parse params
  local inner_rml = ''
  for key, param in pairs(ini) do
    -- set default values
    param.value = saveData[key] or param.value
    saveData[key] = param.value
    -- generate rml
    inner_rml = inner_rml..make_param_rml(funcName, key, param)
  end
  
  if (inner_rml == '') then
    return out
  end
  
  out.document = RocketContext:LoadDocument("System/UI/settings-template.rml")
  out.document:GetElementById('params').inner_rml = inner_rml
  out.document:GetElementById('title').inner_rml = out.info.name
  
  -- load method
  out.load = function(self, saveData)
    if type(saveData) ~= "table" then
      return
    end
    
    local AsInput = Element.As.ElementFormControlInput
    for key, value in pairs(saveData) do
      local element = self.document:GetElementById(key)
      if (element) then
        AsInput(element).value = value
      end
    end
  end
  
  return out
end

return parser